apiVersion: networking.k8s.io/v1beta1
kind: Ingress
metadata:
  name: event-engine-ingrees
  namespace: daas
spec:
  rules:
  - host: {{ host }}
    http:
      paths: {% for service in svc %}
      - backend:
          serviceName: {{ service }}
          servicePort: 8080
        path: /{{ service }} {% endfor %}