global:
  resolve_timeout: 5m
receivers:
- name: default-receiver {% for service in svc %}
- name: {{ service }}-receiver
  webhook_configs:
    - url: "http://{{ host }}/{{ service }}" {% endfor %}
route:
  group_wait: 10s
  group_interval: 5m
  receiver: default-receiver
  repeat_interval: 3h
  routes: {% for service in svc %}
    - receiver: {{ service }}-receiver
      match: {% endfor %}
        